# TO IDEA/EXTENSION: use image and density instead of boxes to visualise the density inside the box
#
# /**
#
# \name{plotDistributions}
#
# \title{Visualise Distributions}
#
# \alias{plotDistributions}
#
# \keyword{}
#
# \description{Boxplot like visualisation of distributions, only the boxes, i.e. the median and roughly the second and third quartile are plotted (cf. \code{boxplot.stats}). The plots may help to identify shortcomings of the raw data or normalised data. The argument \code{quantiles} can be used to visualize two or three arbitrary quantiles with boxes.}
#
# \value{}
#
#
# \arguments{
#  \item{dataMatrix}{data matrix, where columns represent distributions, e.g. raw array data, log(raw array data) or normalised data}
#  \item{transFunc}{data transformation function, e.g. log}
#  \item{quantiles}{missing by default; a vector of two or three increasing quantiles used to determine the boxes to be drawn}
#  \item{title}{plot title, type character string}
#  \item{labels}{vector of names, may substitute column names of \code{dataMatrix}}
#  \item{xAxisLabel}{label for x axis}
#  \item{yAxisLabel}{label for y axis}
#  \item{colourVector}{vector of colours}
#  \item{width}{graphics window width}
#  \item{height}{graphics window height}
#  \item{plotOutput}{character string specifying
#        either "screen" or "pdf"; default: "screen"}
#
# }
# 
# \details{Default of \code{transFunc} is no transformation, i.e. identity. If \code{labels} are supplied at first the column names or secondly a numbering  are used instead.By default the \code{colourVector} is defined as alternating darkred and darkgreen.}
#
# \seealso{\code{boxplot.stats}}
#
# \examples{plotDistributions(cbind(rnorm(100),rnorm(100)), title="Random Gaussians", labels = c("N1","N2"), yAxisLabel="scale")
#          plotDistributions(as.matrix(1:100), quantiles=c(0.25,0.85),title = "Random Gaussians", labels = c("N1"), yAxisLabel = "scale")}
#          plotDistributions(as.matrix(1:100), transFunc=log,quantiles=c(0.1,0.95),title = "Random Gaussians", labels = c("N1"), yAxisLabel = "scale")
#
# \keyword{}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
# */
plotDistributions <- function(dataMatrix, transFunc, quantiles, title, labels, xAxisLabel, yAxisLabel, colourVector, width=8, height=6, plotOutput="screen"){

  

  if( ! plotOutput %in% c("screen","pdf") ){
    warning(" unknown plotOutput in plotDistributions", call.=FALSE)
    return()
  }
  
  if( length(dim(dataMatrix)) != 2 ){
    cat(" dataMatrix must be two dimensional in plotDistributions \n")
    return()
  }
  
  #nrOfItems <- dim(dataMatrix)[1]
  nrOfDist <- dim(dataMatrix)[2]

  if( missing(title) ){
    title <- ""
  }
  
  if( missing(xAxisLabel) ){
    xAxisLabel <- ""
  }
  if( missing(yAxisLabel) ){
    yAxisLabel <- ""
  }
  
  if( missing(transFunc) ){
    transFunc <- function(x) x
  }
  
  if( missing(labels) ){
    dn <- dimnames(dataMatrix)
    if( !is.null(dn[[2]]) ){
      labels <- dn[[2]]
    }else{
      labels <- as.character(1:nr)
    }
  }

  if( missing(colourVector) ){
    colourVector <- rep(c("darkgreen","darkred"),(nrOfDist/2)+1)[1:nrOfDist]
  }

  if( !missing(quantiles) ){
    if( !is.vector(quantiles) ){
      cat("quantiles have to be of type vector in plotDistributions\n")
      return()
    }
    if( length(quantiles) > 3 | length(quantiles) < 2 ){
      cat("quantiles have to be of length two or three in plotDistributions\n")
      return()
    }
    if(! all( sort(quantiles) == quantiles) ){
      cat("quantiles must be increasing in plotDistributions\n")
      return()
    }
    if( min(quantiles) < 0 | max(quantiles) > 1 ){
      cat("quantiles must be in [0,1] in plotDistributions\n")
      return()
    }
  }
  
  BOXCONST <- 0.4 # i.e. 80% of the width is used for the box
  YMARGIN <- 0.1 # i.e. 20% are used for the margin

  if( plotOutput == "screen" ){
    x11(width=width,height=height)
  }else if( plotOutput == "pdf" ){
    pdf(width=width,height=height,file="plotDistributionsOutput.pdf")
  }else{
    stop(" unexpected case; unknown plotOutput in plotDistributions ")
  }
  
  if( missing(quantiles) ){
    boxplotOut <- boxplot(transFunc(dataMatrix) ~ col(dataMatrix), axes=FALSE, plot=FALSE);
    boxStatistics <- boxplotOut$stats
    # roughly first quartile, median and third quartile
    # cf. help(boxplot.stats
    quantileMatrix <- boxStatistics[c(2,3,4),,drop=FALSE]
    dimnames(quantileMatrix) <- list(c("lower","middle","upper"),NULL)
    quantiles <- c(0.25,0.5,0.75)
    names(quantiles) <- c("lower","middle","upper")
  }else{
    quantileMatrix <- as.matrix(apply(transFunc(dataMatrix),2,function(x) quantile(x, probs=quantiles)))
    if( length(quantiles) == 2 ){
      dimnames(quantileMatrix) <- list(c("lower","upper"),NULL)
      names(quantiles) <- c("lower","upper")
    }else if( length(quantiles) == 3 ){
      dimnames(quantileMatrix) <- list(c("lower","middle","upper"),NULL)
      names(quantiles) <- c("lower","middle","upper")
    }else{
      stop("unexpected case in plotDistributions")
    }
  }
  
  minY <- min(quantileMatrix)
  maxY <- max(quantileMatrix)
  yRange <- maxY-minY
  minY <- minY - (YMARGIN*abs(yRange))
  maxY <- maxY + (YMARGIN*abs(yRange))
  plot(x=c(0.5,dim(quantileMatrix)[2]+0.5), y=c(minY,maxY),type="n", axes=FALSE, xlab="",ylab="")
  for(i in 1:dim(quantileMatrix)[2]){
    rect(i-BOXCONST, quantileMatrix["lower",i], i+BOXCONST,quantileMatrix["upper",i], col=colourVector[i])
    if( dim(quantileMatrix)[1] == 3 ){
      lines(x=c(i-BOXCONST, i+BOXCONST), y=c(quantileMatrix["middle",i], quantileMatrix["middle",i]))
    }
  }
  axis(2)
  title(main=title , sub=NULL, xlab=xAxisLabel, ylab=yAxisLabel)
  axis(1,at=1:ncol(dataMatrix),labels=labels)
  mtext( paste("quantiles: ", paste(paste(names(quantiles),quantiles, sep=":"),collapse="; ")) )
  
}# end of function definition
